body.cropduster-upload-form.grp-popup #grp-content {
    top: 0;
}

body.cropduster-upload-form .module, body.cropduster-upload-form .grp-module {
    float: none;
}

body.cropduster-upload-form ul.messagelist,
body.cropduster-upload-form ul.grp-messagelist {
    overflow: hidden;
}

body.cropduster-upload-form #crop-form {
    margin-top: 15px;
}

body.cropduster-upload-form #error-container {
    margin-top: 10px;
}

body.cropduster-upload-form ul.errorlist {
    color: #fff;
}

body.cropduster-upload-form .errornote > ul.errorlist > li {
    overflow: hidden;
    position: relative;
}

body.cropduster-upload-form .errornote .error-field {
    display: block;
    width: 80px;
    position: absolute;
    top: 0;
    left: 0;
}

body.cropduster-upload-form .errornote .error-field:after {
    content: ":";
}
body.cropduster-upload-form .errornote .error-field.error-__all__ {
    display: none;
}
body.cropduster-upload-form .errornote .error-field.error-__all__ + ul.errorlist {
    padding-left: 0;
}

body.cropduster-upload-form ul.errorlist ul.errorlist {
    float: left;
    padding-left: 100px;
    margin: 0;
}

body.cropduster-upload-form .submit-button-container {
    border: 0 !important;
    box-shadow: none !important;
    -webkit-box-shadow: none !important;
    -moz-box-shadow: none !important;
    background: transparent !important;
    -webkit-border-radius: 0 !important;
    -moz-border-radius: 0 !important;
    border-radius: 0 !important;
}

body.cropduster-upload-form .module.footer,
body.cropduster-upload-form .grp-module.grp-footer,
body.cropduster-upload-form .grp-module.grp-fixed-footer {
    border-top: 1px solid #111;
}

body.cropduster-upload-form #current-thumb-info {
    display: none;
    float:left;
    margin-left:5px;
    line-height:27px;
    position:relative;
}
body.cropduster-upload-form #current-thumb-info > div {
    position: absolute;
    top: 0;
    left: 0;
}
body.cropduster-upload-form #current-thumb-info > div#current-thumb-index {
    width:14px;
}
body.cropduster-upload-form #current-thumb-info > div#current-thumb-index {
    position:absolute;
    left:17px;
    width:14px;
}
body.cropduster-upload-form #current-thumb-info > div#current-thumb-index:after {
    content: "/";
    position: absolute;
    left: 17px;
    top: 0;
    width: 5px;
}
body.cropduster-upload-form #current-thumb-info > div#thumb-total-count {
    position: absolute;
    left: 44px;
    top: 0;
    width: 14px;
}
body.cropduster-upload-form #current-thumb-info > div#current-thumb-label {
    left: 70px;
    width: 200px;
    overflow: hidden;
    text-overflow: ellipsis;
    -o-text-overflow: ellipsis;
}
#crop-nav {
    overflow: hidden;
    float: left;
    margin-top: 2px;
    display: none;
}
#thumb-formset-container {
    display: none;
}
form#upload .row.hidden {
    display: none;
}
body.cropduster-debug #thumb-formset-container {
    display: block;
}
body.cropduster-debug form#upload .row.hidden {
    display: block;
}
#nav-left, #nav-right {
    float: left;
    text-align: center;
    text-indent: -10000px;
}
#nav-left span, #nav-right span {
    display: block;
    width: 25px;
    height: 12px;
    background: transparent url(../img/arrows.png) no-repeat 0 0;
}
#nav-left span { background-position: 0 -6px; }
#nav-right span { background-position: -25px -6px; }
#nav-left.disabled span { background-position: 0 -31px; }
#nav-right.disabled span { background-position: -25px -31px; }
#nav-left {
    border-right: 1px solid #000;
}
body.cropduster-upload-form #nav-left.rounded-button,
body.cropduster-upload-form #nav-left.rounded-button:active,
body.cropduster-upload-form #nav-left.rounded-button:hover {
    width: 25px;
    -webkit-border-top-left-radius: 5px;
    -webkit-border-top-right-radius: 0;
    -webkit-border-bottom-left-radius: 5px;
    -webkit-border-bottom-right-radius: 0;
       -moz-border-radius-topleft: 5px;
       -moz-border-radius-bottomleft: 5px;
       -moz-border-radius-topright: 0;
       -moz-border-radius-bottomright: 0;
            border-top-left-radius: 5px;
            border-bottom-left-radius: 5px;
            border-top-right-radius: 0;
            border-bottom-right-radius: 0;
    border-right: 1px solid #000;
    margin: 0;
	padding: 4px 5px 5px 5px !important;
}
body.cropduster-upload-form #nav-right.rounded-button,
body.cropduster-upload-form #nav-right.rounded-button:active,
body.cropduster-upload-form #nav-right.rounded-button:hover {
    width: 25px;
    -webkit-border-top-right-radius: 5px;
    -webkit-border-bottom-right-radius: 5px;
    -webkit-border-top-left-radius: 0;
    -webkit-border-bottom-left-radius: 0;
       -moz-border-radius-topright: 5px;
       -moz-border-radius-bottomright: 5px;
       -moz-border-radius-topleft: 0;
       -moz-border-radius-bottomleft: 0;
            border-top-right-radius: 5px;
            border-bottom-right-radius: 5px;
            border-top-left-radius: 0;
            border-bottom-left-radius: 0;
    margin: 0;
    padding: 4px 5px 5px 5px !important;
}
body.cropduster-upload-form #nav-left.rounded-button:active,
body.cropduster-upload-form #nav-right.rounded-button:active {
    padding: 5px 5px 4px 5px !important;
}

body.cropduster-upload-form h1#step-header {
    margin: 0 0 8px;
    padding: 15px 0 5px;
}

body.cropduster-upload-form #content-main {
    margin-top: 12px;
}

body.cropduster-upload-form form#size .row.width,
body.cropduster-upload-form form#size .row.height {
    display: none;
}

body.cropduster-upload-form form#size .row {
    width: 50%;
    float: left;
    padding: 8px 0;
}
body.cropduster-upload-form form#size .row,
body.cropduster-upload-form form#size .row:first-child,
body.cropduster-upload-form form#size .row:last-child {
    border-top: 1px solid #FFFFFF !important;
    border-bottom: 0 !important;
}

body.cropduster-upload-form form#size .span-4 {
    width: 45px;
}
body.cropduster-upload-form form#size .row:first-child .span-4 {
    padding-left: 15px;
}

body.cropduster-upload-form form#size .span-4 + .span-flexible {
    margin-left: 55px;
}
body.cropduster-upload-form form#size input[type="text"] {
    width: 80px;
}

body.cropduster-upload-form.cropduster-standalone .module.footer {
/*    height: 30px;*/
    display: none;
}
body.cropduster-upload-form.cropduster-standalone #content-main {
    margin-top: 0;
}
body.cropduster-upload-form.cropduster-standalone #content-inner {
    padding: 0;
}
body.cropduster-upload-form.cropduster-standalone form#upload {
    position: relative;
}
body.cropduster-upload-form.cropduster-standalone form#upload #upload-button,
body.cropduster-upload-form.cropduster-standalone form#upload #reupload-button {
    display: none;
    position: absolute;
    top: 5px;
    right: 5px;
}
body.cropduster-upload-form.cropduster-standalone #grp-content #content-inner {
    bottom: 0;
}

/* Django styles without grappelli */
body.cropduster-upload-form #container {
    position: absolute;
    height: 100%;
}
body.cropduster-upload-form #content.colM {
    position: absolute;
    width: 100%;
    height: 100%;
    margin: 0;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    padding: 0 15px 10px 15px;
    box-sizing: border-box;
    -moz-box-sizing: border-box;
}
body.cropduster-upload-form #content.colM footer {
    border: 0;
    border-top: 1px solid #111;
    position: absolute;
    width: auto;
    height: 34px;
    bottom: 0;
    left: 0;
    right: 0;
    margin: 0;
    padding: 2px 5px;
    line-height: 17px;
    color: #fff;
    background: #333;
    background: -moz-linear-gradient(top, #444, #333);
    background: -webkit-gradient(linear, left top, left bottom, from(#444), to(#333));
    background: -o-linear-gradient(top, #444, #333);
    box-sizing: border-box;
    -moz-box-sizing: border-box;
    -webkit-box-sizing: border-box;
}
body.cropduster-upload-form #content.colM footer ul,
body.cropduster-upload-form #content.colM footer ul li {
    margin: 0;
    padding: 0;
    list-style-type: none;
}
body.cropduster-upload-form #content.colM footer ul li {
    float: right;
    margin-left: 10px;
}

/* Help Text: Minimum image size*/
body.cropduster-upload-form #upload-min-size-help {
    float:right;
}
